Method: HashRecursiveMerge#rmerge!
- Defined in:
- lib/spree/core/ext/hash.rb
#rmerge!(other_hash) ⇒ Object
Recursive version of Hash#merge!
Adds the contents of other_hash to hsh, merging entries in hsh with duplicate keys with those from other_hash.
Compared with Hash#merge!, this method supports nested hashes. When both hsh and other_hash contains an entry with the same key, it merges and returns the values from both arrays.
h1 = {"a" => 100, "b" => 200, "c" => {"c1" => 12, "c2" => 14}}
h2 = {"b" => 254, "c" => 300, "c" => {"c1" => 16, "c3" => 94}}
h1.rmerge!(h2) #=> {"a" => 100, "b" => 254, "c" => {"c1" => 16, "c2" => 14, "c3" => 94}}
Simply using Hash#merge! would return
h1.merge!(h2) #=> {"a" => 100, "b" = >254, "c" => {"c1" => 16, "c3" => 94}}
35 36 37 38 39 |
# File 'lib/spree/core/ext/hash.rb', line 35 def rmerge!(other_hash) merge!(other_hash) do |key, oldval, newval| oldval.class == self.class ? oldval.rmerge!(newval) : newval end end |