Module: Spree::Order::Emails
- Extended by:
- ActiveSupport::Concern
- Included in:
- Spree::Order
- Defined in:
- app/models/spree/order/emails.rb
Instance Method Summary collapse
- #deliver_order_confirmation_email ⇒ Object
- #deliver_store_owner_order_notification_email ⇒ Object
-
#deliver_store_owner_order_notification_email? ⇒ Boolean
If you would like to also send confirmation email to store owner(s).
- #send_cancel_email ⇒ Object
Instance Method Details
#deliver_order_confirmation_email ⇒ Object
6 7 8 |
# File 'app/models/spree/order/emails.rb', line 6 def deliver_order_confirmation_email Spree::Deprecation.warn('Spree::Order#deliver_order_confirmation_email is deprecated and will be removed in Spree 5.5. Please create a Subscriber for order.completed event.') end |
#deliver_store_owner_order_notification_email ⇒ Object
15 16 17 |
# File 'app/models/spree/order/emails.rb', line 15 def deliver_store_owner_order_notification_email Spree::Deprecation.warn('Spree::Order#deliver_store_owner_order_notification_email is deprecated and will be removed in Spree 5.5. Please create a Subscriber for order.completed event.') end |
#deliver_store_owner_order_notification_email? ⇒ Boolean
If you would like to also send confirmation email to store owner(s)
11 12 13 |
# File 'app/models/spree/order/emails.rb', line 11 def deliver_store_owner_order_notification_email? false end |
#send_cancel_email ⇒ Object
19 20 21 |
# File 'app/models/spree/order/emails.rb', line 19 def send_cancel_email Spree::Deprecation.warn('Spree::Order#send_cancel_email is deprecated and will be removed in Spree 5.5. Please create a Subscriber for order.canceled event.') end |