Class: Spree::Product
- Inherits:
-
Object
- Object
- Spree::Product
- Includes:
- MemoizedData, Metadata, Metafields, Channels, LegacyMultiStoreSupport, Slugs, Webhooks, ProductScopes, SearchIndexable, TranslatableResource, VendorConcern
- Defined in:
- app/models/spree/product.rb,
app/models/spree/product/slugs.rb,
app/models/spree/product/channels.rb,
app/models/spree/product/webhooks.rb,
app/models/spree/product/legacy_multi_store_support.rb
Defined Under Namespace
Modules: Channels, LegacyMultiStoreSupport, Slugs, Webhooks
Constant Summary collapse
- MEMOIZED_METHODS =
%w[total_on_hand taxonomy_ids taxon_and_ancestors default_variant_id tax_category default_variant variant_for_images brand_taxon main_taxon purchasable? in_stock? backorderable? digital?]
- STATUSES =
%w[draft active archived].freeze
- STATUS_TO_WEBHOOK_EVENT =
{ 'active' => 'activated', 'draft' => 'drafted', 'archived' => 'archived' }.freeze
- TRANSLATABLE_FIELDS =
%i[name description slug meta_description meta_title].freeze
Constants included from Channels
Channels::DEPRECATED_DATE_TO_PUBLICATION_FIELD
Instance Attribute Summary collapse
-
#option_values_hash ⇒ Object
Returns the value of attribute option_values_hash.
-
#prototype_id ⇒ Object
Adding properties and option types on creation based on a chosen prototype.
Class Method Summary collapse
Instance Method Summary collapse
- #any_variant_available?(currency) ⇒ Boolean
- #any_variant_in_stock_or_backorderable? ⇒ Boolean
- #auto_match_taxons ⇒ Object
-
#available? ⇒ Boolean
determine if product is available.
-
#backorderable? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259.
-
#backordered? ⇒ Boolean
determine if any variant (including master) is out of stock and backorderable.
-
#brand_name ⇒ String
Returns the brand name for the product.
-
#brand_taxon ⇒ Spree::Taxon
Returns the brand taxon for the product.
-
#can_supply? ⇒ Boolean
determine if any variant (including master) can be supplied.
-
#category_ids=(ids) ⇒ Object
Maps 6.0 API name (category_ids) to model column (taxon_ids).
-
#default_image ⇒ Object
deprecated
Deprecated.
Use #primary_media instead.
-
#default_variant ⇒ Spree::Variant
Returns default Variant for Product If ‘track_inventory_levels` is enabled it will try to find the first Variant in stock or backorderable, if there’s none it will return first Variant sorted by ‘position` attribute If `track_inventory_levels` is disabled it will return first Variant sorted by `position` attribute.
-
#default_variant_id ⇒ Integer
Returns default Variant ID for Product.
-
#digital? ⇒ Boolean
Check if the product is digital by checking if any of its shipping methods are digital delivery This is used to determine if the product is digital and should have a digital delivery price instead of a physical shipping price.
- #discontinue! ⇒ Object
- #discontinued? ⇒ Boolean
-
#duplicate ⇒ Object
for adding products which are closely related to existing ones define “duplicate_extra” for site-specific actions, eg for additional fields.
-
#ensure_option_types_exist_for_values_hash ⇒ Object
Ensures option_types and product_option_types exist for keys in option_values_hash.
-
#featured_image ⇒ Object
deprecated
Deprecated.
Use #primary_media instead.
- #find_or_build_master ⇒ Object
-
#find_variant_with_images ⇒ Spree::Variant?
Finds first variant with media using preloaded data when available.
- #first_available_variant(currency) ⇒ Object
- #first_or_default_variant(currency) ⇒ Object
-
#gallery_media ⇒ ActiveRecord::Relation
Returns the product’s media gallery.
-
#has_media? ⇒ Boolean
(also: #has_images?, #has_variant_images?)
Returns true if the product has any media (product-level or variant-level).
-
#has_variants? ⇒ Boolean
Checks if product has variants (non-master variants) Uses variant_count counter cache for performance.
-
#image_count ⇒ Object
deprecated
Deprecated.
Use media_count instead
-
#in_stock? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259.
-
#lowest_price(currency) ⇒ Object
returns the lowest price for the product in the given currency prices_including_master are usually already loaded, so this should not trigger an extra query.
- #main_taxon ⇒ Object
-
#master ⇒ Object
Master variant may be deleted (i.e. when the product is deleted) which would make AR’s default finder return nil.
-
#media=(media_params) ⇒ void
Sync media inline.
- #on_sale?(currency) ⇒ Boolean
- #price_varies?(currency) ⇒ Boolean
-
#prices=(prices_params) ⇒ Object
Sets prices on the master variant.
-
#primary_image ⇒ Object
deprecated
Deprecated.
Use #primary_media instead.
-
#purchasable? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259.
-
#secondary_image ⇒ Spree::Asset?
Returns secondary media for Product (for hover effects).
-
#storefront_description ⇒ String
Returns the short description for the product.
-
#tags=(tags) ⇒ Object
Maps tags array to tag_list for API convenience.
-
#tax_category ⇒ Spree::TaxCategory?
Returns tax category for Product.
- #taxons_for_store(store) ⇒ Object
- #to_csv(store = nil) ⇒ Object
- #to_translation_csv(store = nil, locales = []) ⇒ Object
- #total_on_hand ⇒ Object
-
#update_thumbnail! ⇒ Object
Updates primary_media_id to the first media item.
-
#variant_for_images ⇒ Spree::Variant?
Returns the variant that should be used for displaying images.
-
#variants=(variants_params) ⇒ void
Syncs variants from an array of hashes.
-
#variants_and_option_values(current_currency = nil) ⇒ Object
Suitable for displaying only variants that has at least one option value.
Methods included from SearchIndexable
#add_to_search_index, #enqueue_search_index, #enqueue_search_removal, #remove_from_search_index, #search_indexing_enabled?, #search_presentation, #store_ids_for_indexing
Methods included from Channels
#product_publications=, #publication_for
Methods included from Slugs
Methods included from Webhooks
#send_product_activated_webhook, #send_product_archived_webhook, #send_product_drafted_webhook
Methods included from Metadata
#metadata, #metadata=, #public_metadata=
Instance Attribute Details
#option_values_hash ⇒ Object
Returns the value of attribute option_values_hash.
183 184 185 |
# File 'app/models/spree/product.rb', line 183 def option_values_hash @option_values_hash end |
#prototype_id ⇒ Object
Adding properties and option types on creation based on a chosen prototype
466 467 468 |
# File 'app/models/spree/product.rb', line 466 def prototype_id @prototype_id end |
Class Method Details
.bulk_auto_match_taxons(store, product_ids) ⇒ Object
314 315 316 317 318 319 320 321 322 323 324 |
# File 'app/models/spree/product.rb', line 314 def self.bulk_auto_match_taxons(store, product_ids) return if store.taxons.automatic.none? products_to_auto_match_ids = store.products.not_deleted.not_archived.where(id: product_ids).ids auto_match_taxons_jobs = products_to_auto_match_ids.map do |product_id| Spree::Products::AutoMatchTaxonsJob.new(product_id).tap { |job| job.scheduled_at = 30.seconds.from_now } end ActiveJob.perform_all_later(auto_match_taxons_jobs) end |
.like_any(fields, values) ⇒ Object
545 546 547 548 549 550 |
# File 'app/models/spree/product.rb', line 545 def self.like_any(fields, values) conditions = fields.product(values).map do |(field, value)| arel_table[field].matches("%#{value}%") end where conditions.inject(:or) end |
Instance Method Details
#any_variant_available?(currency) ⇒ Boolean
486 487 488 489 490 491 492 |
# File 'app/models/spree/product.rb', line 486 def any_variant_available?(currency) if has_variants? first_available_variant(currency).present? else master.purchasable? && master.price_in(currency).amount.present? end end |
#any_variant_in_stock_or_backorderable? ⇒ Boolean
614 615 616 617 618 619 620 |
# File 'app/models/spree/product.rb', line 614 def any_variant_in_stock_or_backorderable? if has_variants? variants_including_master.in_stock_or_backorderable.exists? else master.in_stock_or_backorderable? end end |
#auto_match_taxons ⇒ Object
631 632 633 634 635 636 637 |
# File 'app/models/spree/product.rb', line 631 def auto_match_taxons return if deleted? return if archived? return if store.nil? || store.taxons.automatic.none? Spree::Products::AutoMatchTaxonsJob.set(wait: 30.seconds).perform_later(id) end |
#available? ⇒ Boolean
determine if product is available. deleted products and products with status different than active are not available
521 522 523 |
# File 'app/models/spree/product.rb', line 521 def available? active? && !deleted? && (available_on.nil? || available_on <= Time.current) end |
#backorderable? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259
337 338 339 |
# File 'app/models/spree/product.rb', line 337 def backorderable? default_variant.backorderable? || variants.any?(&:backorderable?) end |
#backordered? ⇒ Boolean
determine if any variant (including master) is out of stock and backorderable
541 542 543 |
# File 'app/models/spree/product.rb', line 541 def backordered? variants_including_master.any?(&:backordered?) end |
#brand_name ⇒ String
Returns the brand name for the product
598 599 600 |
# File 'app/models/spree/product.rb', line 598 def brand_name brand_taxon&.name end |
#brand_taxon ⇒ Spree::Taxon
Returns the brand taxon for the product
582 583 584 585 586 587 588 589 590 591 592 593 594 |
# File 'app/models/spree/product.rb', line 582 def brand_taxon @brand_taxon ||= if classification_count.zero? nil elsif Spree.use_translations? taxons.joins(:taxonomy). join_translation_table(Taxonomy). find_by(Taxonomy.translation_table_alias => { name: Spree.t(:taxonomy_brands_name) }) elsif taxons.loaded? taxons.find { |taxon| taxon.taxonomy.name == Spree.t(:taxonomy_brands_name) } else taxons.joins(:taxonomy).find_by(Taxonomy.table_name => { name: Spree.t(:taxonomy_brands_name) }) end end |
#can_supply? ⇒ Boolean
determine if any variant (including master) can be supplied
536 537 538 |
# File 'app/models/spree/product.rb', line 536 def can_supply? variants_including_master.any?(&:can_supply?) end |
#category_ids=(ids) ⇒ Object
Maps 6.0 API name (category_ids) to model column (taxon_ids). Accepts both prefixed IDs and raw integer IDs. Only taxons belonging to the product’s own store are assigned — ids from another store’s taxonomies are dropped, preventing cross-store category attachment.
217 218 219 220 221 222 |
# File 'app/models/spree/product.rb', line 217 def category_ids=(ids) decoded_ids = Array(ids).filter_map do |id| id.to_s.include?('_') ? Spree::Taxon.decode_prefixed_id(id) : id end self.taxon_ids = Spree::Taxon.for_store(assignable_store).where(id: decoded_ids).ids end |
#default_image ⇒ Object
Use #primary_media instead.
409 410 411 412 |
# File 'app/models/spree/product.rb', line 409 def default_image Spree::Deprecation.warn('Spree::Product#default_image is deprecated and will be removed in Spree 6.0. Please use Spree::Product#primary_media instead.') primary_media end |
#default_variant ⇒ Spree::Variant
Returns default Variant for Product If ‘track_inventory_levels` is enabled it will try to find the first Variant in stock or backorderable, if there’s none it will return first Variant sorted by ‘position` attribute If `track_inventory_levels` is disabled it will return first Variant sorted by `position` attribute
366 367 368 369 370 371 372 |
# File 'app/models/spree/product.rb', line 366 def default_variant @default_variant ||= if Spree::Config[:track_inventory_levels] && has_variants? && available_variant = variants.detect(&:purchasable?) available_variant else has_variants? ? variants.first : find_or_build_master end end |
#default_variant_id ⇒ Integer
Returns default Variant ID for Product
376 377 378 |
# File 'app/models/spree/product.rb', line 376 def default_variant_id @default_variant_id ||= default_variant.id end |
#digital? ⇒ Boolean
Check if the product is digital by checking if any of its shipping methods are digital delivery This is used to determine if the product is digital and should have a digital delivery price instead of a physical shipping price
627 628 629 |
# File 'app/models/spree/product.rb', line 627 def digital? @digital ||= shipping_category&.includes_digital_shipping_method? end |
#discontinue! ⇒ Object
525 526 527 528 529 |
# File 'app/models/spree/product.rb', line 525 def discontinue! self.discontinue_on = Time.current self.status = 'archived' save(validate: false) end |
#discontinued? ⇒ Boolean
531 532 533 |
# File 'app/models/spree/product.rb', line 531 def discontinued? !!discontinue_on && discontinue_on <= Time.current end |
#duplicate ⇒ Object
for adding products which are closely related to existing ones define “duplicate_extra” for site-specific actions, eg for additional fields
514 515 516 |
# File 'app/models/spree/product.rb', line 514 def duplicate Products::Duplicator.call(product: self) end |
#ensure_option_types_exist_for_values_hash ⇒ Object
Ensures option_types and product_option_types exist for keys in option_values_hash
501 502 503 504 505 506 507 508 509 510 |
# File 'app/models/spree/product.rb', line 501 def ensure_option_types_exist_for_values_hash return if option_values_hash.nil? # we need to convert the keys to string to make it work with UUIDs required_option_type_ids = option_values_hash.keys.map(&:to_s) missing_option_type_ids = required_option_type_ids - option_type_ids.map(&:to_s) missing_option_type_ids.each do |id| product_option_types.create(option_type_id: id) end end |
#featured_image ⇒ Object
Use #primary_media instead.
415 416 417 418 |
# File 'app/models/spree/product.rb', line 415 def featured_image Spree::Deprecation.warn('Spree::Product#featured_image is deprecated and will be removed in Spree 6.0. Please use Spree::Product#primary_media instead.') primary_media end |
#find_or_build_master ⇒ Object
345 346 347 |
# File 'app/models/spree/product.rb', line 345 def find_or_build_master master || build_master end |
#find_variant_with_images ⇒ Spree::Variant?
Finds first variant with media using preloaded data when available.
447 448 449 450 451 |
# File 'app/models/spree/product.rb', line 447 def find_variant_with_images return variants.find(&:has_media?) if variants.loaded? variants.joins(:images).first end |
#first_available_variant(currency) ⇒ Object
478 479 480 |
# File 'app/models/spree/product.rb', line 478 def first_available_variant(currency) variants.find { |v| v.purchasable? && v.price_in(currency).amount.present? } end |
#first_or_default_variant(currency) ⇒ Object
468 469 470 471 472 473 474 475 476 |
# File 'app/models/spree/product.rb', line 468 def first_or_default_variant(currency) if !has_variants? default_variant elsif first_available_variant(currency).present? first_available_variant(currency) else variants.first end end |
#gallery_media ⇒ ActiveRecord::Relation
Returns the product’s media gallery. Uses product-level media if present, otherwise falls back to variant images.
383 384 385 386 387 |
# File 'app/models/spree/product.rb', line 383 def gallery_media return media if association(:media).loaded? ? media.any? : media.exists? variant_images end |
#has_media? ⇒ Boolean Also known as: has_images?, has_variant_images?
Returns true if the product has any media (product-level or variant-level). Uses counter cache for performance.
392 393 394 395 396 |
# File 'app/models/spree/product.rb', line 392 def has_media? return variant_images.any? if association(:variant_images).loaded? media_count.positive? end |
#has_variants? ⇒ Boolean
Checks if product has variants (non-master variants) Uses variant_count counter cache for performance
352 353 354 355 356 |
# File 'app/models/spree/product.rb', line 352 def has_variants? return variants.size.positive? if variants.loaded? variant_count.positive? end |
#image_count ⇒ Object
Use media_count instead
433 434 435 |
# File 'app/models/spree/product.rb', line 433 def image_count media_count end |
#in_stock? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259
332 333 334 |
# File 'app/models/spree/product.rb', line 332 def in_stock? @in_stock ||= default_variant.in_stock? || variants.any?(&:in_stock?) end |
#lowest_price(currency) ⇒ Object
returns the lowest price for the product in the given currency prices_including_master are usually already loaded, so this should not trigger an extra query
496 497 498 |
# File 'app/models/spree/product.rb', line 496 def lowest_price(currency) prices_including_master.find_all { |p| p.currency == currency }.min_by(&:amount) end |
#main_taxon ⇒ Object
602 603 604 605 606 |
# File 'app/models/spree/product.rb', line 602 def main_taxon return if classification_count.zero? @main_taxon ||= taxons.first end |
#master ⇒ Object
Master variant may be deleted (i.e. when the product is deleted) which would make AR’s default finder return nil. This is a stopgap for that little problem.
576 577 578 |
# File 'app/models/spree/product.rb', line 576 def master super || variants_including_master.with_deleted.find_by(is_master: true) end |
#media=(media_params) ⇒ void
This method returns an undefined value.
Sync media inline. Entries with ‘id` patch the existing asset (alt/position/variant_ids); entries with `signed_id` create + attach a fresh upload; missing items are left alone (delete still goes through the dedicated DELETE /media endpoint to avoid accidental data loss when a form ships stale state).
Deferred: ActiveStorage attaches require a persisted record, so on new records we stash the params and replay them in ‘after_create`.
234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 |
# File 'app/models/spree/product.rb', line 234 def media=(media_params) # Blank input is a no-op — never call `super` with an empty array, # because the ActiveRecord collection setter would replace media with # `[]` and trigger `dependent: :destroy` on every persisted asset. # Explicit deletes go through the dedicated DELETE /media endpoint. return if media_params.blank? return super if media_params.first.is_a?(Spree::Asset) if new_record? @pending_media_params = media_params return end apply_media(media_params) end |
#on_sale?(currency) ⇒ Boolean
341 342 343 |
# File 'app/models/spree/product.rb', line 341 def on_sale?(currency) prices_including_master.find_all { |p| p.currency == currency }.any?(&:discounted?) end |
#price_varies?(currency) ⇒ Boolean
482 483 484 |
# File 'app/models/spree/product.rb', line 482 def price_varies?(currency) prices_including_master.find_all { |p| p.currency == currency && p.amount.present? }.map(&:amount).uniq.count > 1 end |
#prices=(prices_params) ⇒ Object
Sets prices on the master variant. Accepts array of { currency:, amount:, compare_at_amount: } hashes.
209 210 211 |
# File 'app/models/spree/product.rb', line 209 def prices=(prices_params) find_or_build_master.prices = prices_params end |
#primary_image ⇒ Object
Use #primary_media instead.
421 422 423 424 |
# File 'app/models/spree/product.rb', line 421 def primary_image Spree::Deprecation.warn('Spree::Product#primary_image is deprecated and will be removed in Spree 6.0. Please use Spree::Product#primary_media instead.') primary_media end |
#purchasable? ⇒ Boolean
Can’t use short form block syntax due to github.com/Netflix/fast_jsonapi/issues/259
327 328 329 |
# File 'app/models/spree/product.rb', line 327 def purchasable? @purchasable ||= default_variant.purchasable? || variants.any?(&:purchasable?) end |
#secondary_image ⇒ Spree::Asset?
Returns secondary media for Product (for hover effects).
428 429 430 |
# File 'app/models/spree/product.rb', line 428 def secondary_image variant_for_images&.secondary_image end |
#storefront_description ⇒ String
Returns the short description for the product
455 456 457 |
# File 'app/models/spree/product.rb', line 455 def storefront_description description end |
#tags=(tags) ⇒ Object
Maps tags array to tag_list for API convenience.
203 204 205 |
# File 'app/models/spree/product.rb', line 203 def () self.tag_list = end |
#tax_category ⇒ Spree::TaxCategory?
Returns tax category for Product
461 462 463 |
# File 'app/models/spree/product.rb', line 461 def tax_category @tax_category ||= super || TaxCategory.default end |
#taxons_for_store(store) ⇒ Object
608 609 610 611 612 |
# File 'app/models/spree/product.rb', line 608 def taxons_for_store(store) return if classification_count.zero? taxons.loaded? ? taxons.find_all { |taxon| taxon.taxonomy.store_id == store.id } : taxons.for_store(store) end |
#to_csv(store = nil) ⇒ Object
639 640 641 642 643 644 645 646 647 648 649 650 651 652 653 654 655 656 657 658 659 660 661 662 663 664 665 666 667 668 669 670 671 672 673 674 675 676 677 678 679 |
# File 'app/models/spree/product.rb', line 639 def to_csv(store = nil) store ||= self.store properties_for_csv = if respond_to?(:product_properties) && Spree::Config.respond_to?(:product_properties_enabled) && Spree::Config[:product_properties_enabled] Spree::Property.order(:position).flat_map do |property| [ property.name, product_properties.find { |pp| pp.property_id == property.id }&.value ] end else [] end ||= Spree::MetafieldDefinition.for_resource_type('Spree::Product').order(:namespace, :key).map do |mf_def| .find { |mf| mf. == mf_def.id }&.csv_value end taxons_for_csv ||= taxons.manual.reorder(depth: :desc).first(3).pluck(:pretty_name) taxons_for_csv.fill(nil, taxons_for_csv.size...3) csv_lines = [] all_variants = has_variants? ? variants_including_master.to_a : [master] default_currency = store.default_currency additional_currencies = store.supported_currencies_list.map(&:iso_code) - [default_currency] # Primary rows in the store's default currency all_variants.each_with_index do |variant, index| csv_lines << Spree::CSV::ProductVariantPresenter.new(self, variant, index, properties_for_csv, taxons_for_csv, store, ).call end # Price-only rows for each additional currency additional_currencies.each do |currency| all_variants.each do |variant| next unless variant.amount_in(currency) csv_lines << Spree::CSV::ProductVariantPresenter.new(self, variant, 0, [], [], store, [], currency).call end end csv_lines end |
#to_translation_csv(store = nil, locales = []) ⇒ Object
681 682 683 684 685 686 687 688 689 690 |
# File 'app/models/spree/product.rb', line 681 def to_translation_csv(store = nil, locales = []) locales.filter_map do |locale| # Only export if at least one field has a translation for this locale has_translation = Spree::CSV::ProductTranslationPresenter::TRANSLATABLE_FIELDS.any? do |field| get_field_with_locale(locale, field).present? end Spree::CSV::ProductTranslationPresenter.new(self, locale).call if has_translation end end |
#total_on_hand ⇒ Object
561 562 563 564 565 566 567 568 569 570 571 |
# File 'app/models/spree/product.rb', line 561 def total_on_hand @total_on_hand ||= if any_variants_not_track_inventory? BigDecimal::INFINITY else if variants_including_master.loaded? variants_including_master.sum(&:total_on_hand) else stock_items.loaded? ? stock_items.sum(&:count_on_hand) : stock_items.sum(:count_on_hand) end end end |
#update_thumbnail! ⇒ Object
Updates primary_media_id to the first media item. Checks product-level media first, then falls back to variant images. Called when media is added, removed, or reordered.
440 441 442 443 |
# File 'app/models/spree/product.rb', line 440 def update_thumbnail! first_media = media.order(:position).first || variant_images.order(:position).first update_column(:primary_media_id, first_media&.id) end |
#variant_for_images ⇒ Spree::Variant?
Returns the variant that should be used for displaying images. Priority: master > default_variant > first variant with images
404 405 406 |
# File 'app/models/spree/product.rb', line 404 def variant_for_images @variant_for_images ||= find_variant_for_images end |
#variants=(variants_params) ⇒ void
This method returns an undefined value.
Syncs variants from an array of hashes. Creates new variants, updates existing ones (matched by :id), and removes unlisted ones. Must be called on a persisted product (use after_save or call explicitly after create).
255 256 257 258 259 260 261 262 263 264 265 |
# File 'app/models/spree/product.rb', line 255 def variants=(variants_params) return super if variants_params.blank? || variants_params.first.is_a?(Spree::Variant) # Store for deferred processing if product is not yet persisted if new_record? @pending_variants_params = variants_params return end apply_variants(variants_params) end |
#variants_and_option_values(current_currency = nil) ⇒ Object
Suitable for displaying only variants that has at least one option value. There may be scenarios where an option type is removed and along with it all option values. At that point all variants associated with only those values should not be displayed to frontend users. Otherwise it breaks the idea of having variants
557 558 559 |
# File 'app/models/spree/product.rb', line 557 def variants_and_option_values(current_currency = nil) variants.active(current_currency).joins(:option_value_variants) end |