Class: Spree::Shipment

Inherits:
Object
  • Object
show all
Extended by:
DisplayMoney
Includes:
Metadata, Metafields, NumberIdentifier, Spree::Security::Shipments, CustomEvents, Webhooks, VendorConcern
Defined in:
app/models/spree/shipment.rb,
app/models/spree/shipment/webhooks.rb,
app/models/spree/shipment/custom_events.rb

Defined Under Namespace

Modules: CustomEvents, Webhooks Classes: ManifestItem

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from DisplayMoney

money_methods

Methods included from CustomEvents

#publish_shipment_canceled_event, #publish_shipment_resumed_event, #publish_shipment_shipped_event

Methods included from Webhooks

#send_shipment_shipped_webhook

Methods included from Metadata

#metadata, #metadata=, #public_metadata=

Instance Attribute Details

#special_instructionsObject

Returns the value of attribute special_instructions.



44
45
46
# File 'app/models/spree/shipment.rb', line 44

def special_instructions
  @special_instructions
end

Instance Method Details

#add_shipping_method(shipping_method, selected = false) ⇒ Object



137
138
139
# File 'app/models/spree/shipment.rb', line 137

def add_shipping_method(shipping_method, selected = false)
  shipping_rates.create(shipping_method: shipping_method, selected: selected, cost: cost)
end

#after_cancelObject



141
142
143
# File 'app/models/spree/shipment.rb', line 141

def after_cancel
  manifest.each { |item| manifest_restock(item) }
end

#after_resumeObject



145
146
147
# File 'app/models/spree/shipment.rb', line 145

def after_resume
  manifest.each { |item| manifest_unstock(item) }
end

#amountObject



129
130
131
# File 'app/models/spree/shipment.rb', line 129

def amount
  cost
end

#backordered?Boolean

Returns true if the shipment has any backordered inventory units

Returns:



152
153
154
# File 'app/models/spree/shipment.rb', line 152

def backordered?
  inventory_units.any?(&:backordered?)
end

#determine_state(order) ⇒ Object

Determines the appropriate state according to the following logic:

pending unless order is complete and order.payment_state is paid shipped if already shipped (ie. does not change the state) ready all other cases



185
186
187
188
189
190
191
192
# File 'app/models/spree/shipment.rb', line 185

def determine_state(order)
  return 'canceled' if canceled? || order.canceled?
  return 'pending' unless order.can_ship?
  return 'pending' if inventory_units.any?(&:backordered?)
  return 'shipped' if shipped?

  order.paid? || Spree::Config[:auto_capture_on_dispatch] ? 'ready' : 'pending'
end

#digital?Boolean

Returns:



133
134
135
# File 'app/models/spree/shipment.rb', line 133

def digital?
  shipping_method&.digital? || false
end

#discounted_costObject Also known as: discounted_amount



194
195
196
# File 'app/models/spree/shipment.rb', line 194

def discounted_cost
  cost + promo_total
end

#final_priceObject Also known as: total



199
200
201
# File 'app/models/spree/shipment.rb', line 199

def final_price
  cost + adjustment_total
end

#final_price_with_itemsObject



205
206
207
# File 'app/models/spree/shipment.rb', line 205

def final_price_with_items
  item_cost + final_price
end

#finalize!Object



223
224
225
226
# File 'app/models/spree/shipment.rb', line 223

def finalize!
  inventory_units.finalize_units!
  after_resume
end

#free?Boolean

Returns:



209
210
211
212
213
# File 'app/models/spree/shipment.rb', line 209

def free?
  return true if final_price == BigDecimal(0)

  with_free_shipping_promotion?
end

#include?(variant) ⇒ Boolean

Returns:



228
229
230
# File 'app/models/spree/shipment.rb', line 228

def include?(variant)
  inventory_units_for(variant).present?
end

#inventory_units_for(variant) ⇒ Object



232
233
234
# File 'app/models/spree/shipment.rb', line 232

def inventory_units_for(variant)
  inventory_units.where(variant_id: variant.id)
end

#inventory_units_for_item(line_item, variant = nil) ⇒ Object



236
237
238
# File 'app/models/spree/shipment.rb', line 236

def inventory_units_for_item(line_item, variant = nil)
  inventory_units.where(line_item_id: line_item.id, variant_id: line_item.variant_id || variant.id)
end

#item_costBigDecimal

Returns the cost of the shipment

Returns:

  • (BigDecimal)


248
249
250
# File 'app/models/spree/shipment.rb', line 248

def item_cost
  manifest.map { |m| (m.line_item.price + (m.line_item.adjustment_total / m.line_item.quantity)) * m.quantity }.sum
end

#item_quantityObject

Returns the total quantity of all line items in the shipment



241
242
243
# File 'app/models/spree/shipment.rb', line 241

def item_quantity
  manifest.sum(&:quantity)
end

#item_weightBigDecimal

Returns the weight of the shipment

Returns:

  • (BigDecimal)


255
256
257
# File 'app/models/spree/shipment.rb', line 255

def item_weight
  manifest.map { |m| m.line_item.item_weight }.sum
end

#line_itemsObject



266
267
268
# File 'app/models/spree/shipment.rb', line 266

def line_items
  inventory_units.includes(:line_item).map(&:line_item).uniq
end

#manifestObject



272
273
274
275
276
277
278
279
280
281
282
283
284
285
# File 'app/models/spree/shipment.rb', line 272

def manifest
  # Grouping by the ID means that we don't have to call out to the association accessor
  # This makes the grouping by faster because it results in less SQL cache hits.
  inventory_units.group_by(&:variant_id).map do |_variant_id, units|
    units.group_by(&:line_item_id).map do |_line_item_id, units|
      states = {}
      units.group_by(&:state).each { |state, iu| states[state] = iu.sum(&:quantity) }

      line_item = units.first.line_item
      variant = units.first.variant
      ManifestItem.new(line_item, variant, units.sum(&:quantity), states)
    end
  end.flatten
end

#nameString

Returns the shipment number and shipping method name

Returns:

  • (String)


125
126
127
# File 'app/models/spree/shipment.rb', line 125

def name
  [number, shipping_method&.name].compact.join(' ').strip
end

#partial?Boolean

Returns true if not all of the shipment’s line items are fully shipped

Returns:



173
174
175
176
177
178
# File 'app/models/spree/shipment.rb', line 173

def partial?
  manifest.any? do |manifest_item|
    line_item = manifest_item.line_item
    line_item.quantity > manifest_item.quantity
  end
end

#process_order_paymentsObject



287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
# File 'app/models/spree/shipment.rb', line 287

def process_order_payments
  pending_payments = order.pending_payments.
                     sort_by(&:uncaptured_amount).reverse

  shipment_to_pay = final_price_with_items
  payments_amount = 0

  payments_pool = pending_payments.each_with_object([]) do |payment, pool|
    break if payments_amount >= shipment_to_pay

    payments_amount += payment.uncaptured_amount
    pool << payment
  end

  payments_pool.each do |payment|
    capturable_amount = if payment.amount >= shipment_to_pay
                          shipment_to_pay
                        else
                          payment.amount
                        end

    cents = (capturable_amount * 100).to_i
    payment.capture!(cents)
    shipment_to_pay -= capturable_amount
  end
end

#ready_or_pending?Boolean

Returns:



314
315
316
# File 'app/models/spree/shipment.rb', line 314

def ready_or_pending?
  ready? || pending?
end

#refresh_rates(shipping_method_filter = ShippingMethod::DISPLAY_ON_FRONT_END) ⇒ Object



318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
# File 'app/models/spree/shipment.rb', line 318

def refresh_rates(shipping_method_filter = ShippingMethod::DISPLAY_ON_FRONT_END)
  return shipping_rates if shipped?
  return [] unless can_get_rates?

  # StockEstimator.new assignment below will replace the current shipping_method
  original_shipping_method_id = shipping_method.try(:id)

  self.shipping_rates = Stock::Estimator.new(order).
                        shipping_rates(to_package, shipping_method_filter)

  if shipping_method
    selected_rate = shipping_rates.detect do |rate|
      if original_shipping_method_id
        rate.shipping_method_id == original_shipping_method_id
      else
        rate.selected
      end
    end
    save!
    self.selected_shipping_rate_id = selected_rate.id if selected_rate
    reload
  end

  shipping_rates
end

#selected_shipping_rate_idObject



344
345
346
# File 'app/models/spree/shipment.rb', line 344

def selected_shipping_rate_id
  selected_shipping_rate.try(:id)
end

#selected_shipping_rate_id=(id) ⇒ Object



348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
# File 'app/models/spree/shipment.rb', line 348

def selected_shipping_rate_id=(id)
  # Explicitly updates the timestamp in order to bust cache dependent on "updated_at"
  shipping_rates.update_all(selected: false, updated_at: Time.current)
  shipping_rates.update(id, selected: true)
  save!
  # Reload associations to pick up the new selected shipping rate
  shipping_rates.reset
  association(:selected_shipping_rate).reset
  # Update shipment cost and order totals only for incomplete orders (during checkout)
  # For completed orders, totals are managed separately (e.g., in tests or admin adjustments)
  return if order.completed?

  update_amounts
  reload # reload to pick up cost set by update_columns in update_amounts
  order.set_shipments_cost
end

#set_up_inventory(state, variant, order, line_item, quantity = 1) ⇒ Object



365
366
367
368
369
370
371
372
373
374
375
# File 'app/models/spree/shipment.rb', line 365

def set_up_inventory(state, variant, order, line_item, quantity = 1)
  return if quantity <= 0

  inventory_units.create(
    state: state,
    variant_id: variant.id,
    order_id: order.id,
    line_item_id: line_item.id,
    quantity: quantity
  )
end

#shippable?Boolean

Returns true if the shipment is shippable

Returns:



166
167
168
# File 'app/models/spree/shipment.rb', line 166

def shippable?
  can_ship? && (tracked? || digital?)
end

#shipped=(value) ⇒ Object



377
378
379
380
381
# File 'app/models/spree/shipment.rb', line 377

def shipped=(value)
  return unless value == '1' && shipped_at.nil?

  self.shipped_at = Time.current
end

#shipping_methodSpree::ShippingMethod

Returns the shipping method of the selected shipping rate



386
387
388
# File 'app/models/spree/shipment.rb', line 386

def shipping_method
  selected_shipping_rate&.shipping_method || shipping_rates.first&.shipping_method
end

#tax_categorySpree::TaxCategory

Returns the tax category of the selected shipping rate

Returns:



393
394
395
# File 'app/models/spree/shipment.rb', line 393

def tax_category
  selected_shipping_rate.try(:tax_rate).try(:tax_category)
end

#tax_category_idInteger

Returns the tax category ID of the selected shipping rate

Returns:

  • (Integer)


400
401
402
# File 'app/models/spree/shipment.rb', line 400

def tax_category_id
  selected_shipping_rate.try(:tax_rate).try(:tax_category_id)
end

#tax_totalObject

Only one of either included_tax_total or additional_tax_total is set This method returns the total of the two. Saves having to check if tax is included or additional.



407
408
409
# File 'app/models/spree/shipment.rb', line 407

def tax_total
  included_tax_total + additional_tax_total
end

#to_packageObject



411
412
413
414
415
416
417
# File 'app/models/spree/shipment.rb', line 411

def to_package
  package = Stock::Package.new(stock_location)
  inventory_units.includes(:variant).joins(:variant).group_by(&:state).each do |state, state_inventory_units|
    package.add_multiple state_inventory_units, state.to_sym
  end
  package
end

#tracked?Boolean

Returns true if the shipment is tracked

Returns:



159
160
161
# File 'app/models/spree/shipment.rb', line 159

def tracked?
  tracking.present? || tracking_url.present?
end

#tracking_urlObject



419
420
421
# File 'app/models/spree/shipment.rb', line 419

def tracking_url
  @tracking_url ||= shipping_method&.build_tracking_url(tracking)
end

#transfer_to_location(variant, quantity, stock_location) ⇒ Object



450
451
452
453
454
455
456
# File 'app/models/spree/shipment.rb', line 450

def transfer_to_location(variant, quantity, stock_location)
  transfer_to_shipment(
    variant,
    quantity,
    order.shipments.build(stock_location: stock_location)
  )
end

#transfer_to_shipment(variant, quantity, shipment_to_transfer_to) ⇒ Object



458
459
460
461
462
463
464
465
466
467
# File 'app/models/spree/shipment.rb', line 458

def transfer_to_shipment(variant, quantity, shipment_to_transfer_to)
  Spree::FulfilmentChanger.new(
    current_stock_location: stock_location,
    desired_stock_location: shipment_to_transfer_to.stock_location,
    current_shipment: self,
    desired_shipment: shipment_to_transfer_to,
    variant: variant,
    quantity: quantity
  )
end

#update!(order) ⇒ Object

Updates various aspects of the Shipment while bypassing any callbacks. Note that this method takes an explicit reference to the Order object. This is necessary because the association actually has a stale (and unsaved) copy of the Order and so it will not yield the correct results.



440
441
442
443
444
445
446
447
448
# File 'app/models/spree/shipment.rb', line 440

def update!(order)
  old_state = state
  new_state = determine_state(order)
  update_columns(
    state: new_state,
    updated_at: Time.current
  )
  after_ship if new_state == 'shipped' && old_state != 'shipped'
end

#update_amountsObject



423
424
425
426
427
428
429
430
431
# File 'app/models/spree/shipment.rb', line 423

def update_amounts
  if selected_shipping_rate
    update_columns(
      cost: selected_shipping_rate.cost,
      adjustment_total: adjustments.additional.map(&:update!).compact.sum,
      updated_at: Time.current
    )
  end
end

#update_attributes_and_order(params = {}) ⇒ Object



433
434
435
# File 'app/models/spree/shipment.rb', line 433

def update_attributes_and_order(params = {})
  Shipments::Update.call(shipment: self, shipment_attributes: params).success?
end

#weight_unitString

Returns the weight unit of the shipment

Returns:

  • (String)


262
263
264
# File 'app/models/spree/shipment.rb', line 262

def weight_unit
  manifest.first.line_item.weight_unit
end

#with_free_shipping_promotion?Boolean

Returns true if the shipment has a free shipping promotion applied

Returns:



218
219
220
221
# File 'app/models/spree/shipment.rb', line 218

def with_free_shipping_promotion?
  adjustments.promotion.joins("INNER JOIN #{Spree::PromotionAction.table_name} ON #{Spree::PromotionAction.table_name}.id = #{Spree::Adjustment.table_name}.source_id").
    where("#{Spree::PromotionAction.table_name}.type = 'Spree::Promotion::Actions::FreeShipping'").exists?
end